<?php
/**
 * Theme sprecific functions and definitions
 */


/* Theme setup section
------------------------------------------------------------------- */

// Set the content width based on the theme's design and stylesheet.
if ( ! isset( $content_width ) ) $content_width = 1170; /* pixels */

// Add theme specific actions and filters
// Attention! Function were add theme specific actions and filters handlers must have priority 1
if ( !function_exists( 'themerex_theme_setup' ) ) {
	add_action( 'themerex_action_before_init_theme', 'themerex_theme_setup', 1 );
	function themerex_theme_setup() {

		// Add default posts and comments RSS feed links to head
		add_theme_support( 'automatic-feed-links' );

		// Enable support for Post Thumbnails
		add_theme_support( 'post-thumbnails' );

		// Custom header setup
		add_theme_support( 'custom-header', array('header-text'=>false));

		// Custom backgrounds setup
		add_theme_support( 'custom-background');

		// Supported posts formats
		add_theme_support( 'post-formats', array('gallery', 'video', 'audio', 'link', 'quote', 'image', 'status', 'aside', 'chat') );

		// Autogenerate title tag
		add_theme_support('title-tag');

		// Add user menu
		add_theme_support('nav-menus');

		// WooCommerce Support
		add_theme_support( 'woocommerce' );

		// Register theme menus
		add_filter( 'themerex_filter_add_theme_menus',		'themerex_add_theme_menus' );

		// Register theme sidebars
		add_filter( 'themerex_filter_add_theme_sidebars',	'themerex_add_theme_sidebars' );

		// Set theme name and folder (for the update notifier)
		add_filter('themerex_filter_update_notifier', 		'themerex_set_theme_names_for_updater');

		add_filter( 'themerex_filter_required_plugins',		'themerex_add_required_plugins' );

		add_action('wp_head', 'themerex_head_add_page_meta', 1);

		// Add wide and full blocks support
		add_theme_support( 'align-wide' );
		
		// Set list of the theme required custom fonts from folder /css/font-faces
		// Attention! Font's folder must have name equal to the font's name
		themerex_set_global('required_custom_fonts', array(
			'Amadeus'
			)
		);
	}
}


// Add/Remove theme nav menus
if ( !function_exists( 'themerex_add_theme_menus' ) ) {
	//Handler of add_filter( 'themerex_action_add_theme_menus', 'themerex_add_theme_menus' );
	function themerex_add_theme_menus($menus) {
		
		if (isset($menus['menu_side'])) unset($menus['menu_side']);
		return $menus;
	}
}


// Add theme specific widgetized areas
if ( !function_exists( 'themerex_add_theme_sidebars' ) ) {
	//Handler of add_filter( 'themerex_filter_add_theme_sidebars',	'themerex_add_theme_sidebars' );
	function themerex_add_theme_sidebars($sidebars=array()) {
		if (is_array($sidebars)) {
			$theme_sidebars = array(
				'sidebar_main'		=> esc_html__( 'Main Sidebar', 'alliance' ),
				'sidebar_footer'	=> esc_html__( 'Footer Sidebar', 'alliance' )
			);
			if (themerex_exists_woocommerce()) {
				$theme_sidebars['sidebar_cart']  = esc_html__( 'WooCommerce Cart Sidebar', 'alliance' );
			}
			$sidebars = array_merge($theme_sidebars, $sidebars);
		}
		return $sidebars;
	}
}

// Set theme name and folder (for the update notifier)
if ( !function_exists( 'themerex_set_theme_names_for_updater' ) ) {
	//Handler of add_filter('themerex_filter_update_notifier', 'themerex_set_theme_names_for_updater');
	function themerex_set_theme_names_for_updater($opt) {
		$opt['theme_name']   = 'Alliance';
		$opt['theme_folder'] = 'alliance';
		return $opt;
	}
}

// Custom login
if ( !function_exists( 'themerex_custom_login' ) ) {
	add_action('login_head', 'themerex_custom_login');
	function themerex_custom_login() {
		// Include CSS 
		wp_enqueue_style ( 'custom-login', themerex_get_file_url('css/custom-login-styles.css'), array(), null );
	}
}

//reset password message
function reset_password_message( $message, $key ) {

	if ( strpos($_POST['user_login'], '@') ) {
		$user_data = get_user_by('email', themerex_get_value_gp('user_login'));
	} else {
		$login = themerex_get_value_gp('user_login');
		$user_data = get_user_by('login', $login);
	}

	$user_login = $user_data->user_login;

	$msg = esc_html__('The password for the following account has been requested to be reset:', 'alliance' ). "\r\n\r\n";
	$msg .= network_site_url() . "\r\n\r\n";
	$msg .= sprintf(esc_html__('Username: %s', 'alliance' ), $user_login) . "\r\n\r\n";
	$msg .= esc_html__('If this message was sent in error, please ignore this email.', 'alliance' ) . "\r\n\r\n";
	$msg .= esc_html__('To reset your password, visit the following address: ', 'alliance' );
	$msg .= network_site_url("wp-login.php?action=rp&key=$key&login=" . rawurlencode($user_login), 'login') ."\r\n";

	return $msg;

}

add_filter('retrieve_password_message', 'reset_password_message', null, 2);

// Add theme required plugins
if ( !function_exists( 'themerex_add_required_plugins' ) ) {
	//Handler of add_filter( 'themerex_filter_required_plugins',		'themerex_add_required_plugins' );
	function themerex_add_required_plugins($plugins) {
		$plugins[] = array(
			'name' 		=> esc_html__('Universal Services Plugin', 'alliance'),
			'version'	=> '2.3.1',					// Minimal required version
			'slug' 		=> 'universal_services_plugin',
			'source'	=> themerex_get_file_dir('plugins/universal_services_plugin.zip'),
			'required' 	=> true
		);
		return $plugins;
	}
}


// Add page meta to the head
if (!function_exists('themerex_head_add_page_meta')) {
	//Handler of add_action('wp_head', 'themerex_head_add_page_meta', 1);
	function themerex_head_add_page_meta() { ?>
		<meta charset="<?php bloginfo( 'charset' ); ?>" />
		<?php
		if (themerex_get_theme_option('responsive_layouts') == 'yes') {
			?>
			<meta name="viewport" content="width=device-width, initial-scale=1, maximum-scale=1">
			<?php
		}
		if (floatval(get_bloginfo('version')) < "4.1") {
			?>
			<title><?php wp_title( '|', true, 'right' ); ?></title>
			<?php
		}
		?>
			<link rel="profile" href="http://gmpg.org/xfn/11" />
			<link rel="pingback" href="<?php bloginfo( 'pingback_url' ); ?>" />
			<?php
		$favicon = themerex_get_custom_option('favicon');
		$theme_skin = sanitize_file_name('alliance');
		if (!$favicon) {
			if ( file_exists(themerex_get_file_dir('skins/'.($theme_skin).'/images/favicon.ico')) )
				$favicon = themerex_get_file_url('skins/'.($theme_skin).'/images/favicon.ico');
			if ( !$favicon && file_exists(themerex_get_file_dir('favicon.ico')) )
				$favicon = themerex_get_file_url('favicon.ico');
		}
		if ($favicon) {
			?>
			<link rel="icon" type="image/x-icon" href="<?php echo esc_url($favicon); ?>" />
			<?php
		}
	}
}

function hex2rgb1($hex) {
	$hex = str_replace("#", "", $hex);

	if(strlen($hex) == 3) {
		$r = hexdec(substr($hex,0,1).substr($hex,0,1));
		$g = hexdec(substr($hex,1,1).substr($hex,1,1));
		$b = hexdec(substr($hex,2,1).substr($hex,2,1));
	} else {
		$r = hexdec(substr($hex,0,2));
		$g = hexdec(substr($hex,2,2));
		$b = hexdec(substr($hex,4,2));
	}
	$rgb = array($r, $g, $b);
	return $rgb; // returns an array with the rgb values
}


// Check if plugin 'WPBakery PageBuilder' (old name is 'WPBakery PageBuilder') is installed and activated
if ( !function_exists( 'themerex_exists_vc' ) ) {
	function themerex_exists_vc() {
		return class_exists('Vc_Manager');
	}
}

// Check if plugin 'Elementor' is installed and activated
if ( !function_exists( 'themerex_exists_elementor' ) ) {
	function themerex_exists_elementor() {
		return class_exists('Elementor\Plugin');
	}
}

// Check if plugin 'Gutenberg' is installed and activated
if ( !function_exists( 'themerex_exists_gutenberg' ) ) {
	function themerex_exists_gutenberg() {
		return function_exists( 'the_gutenberg_project' ) && function_exists( 'register_block_type' );
	}
}

// Add theme required plugins
if ( !function_exists( 'themerex_add_trx_addons' ) ) {
	add_filter( 'trx_addons_active', 'themerex_add_trx_addons' );
	function themerex_add_trx_addons($enable=true) {
		return true;
	}
}

//------------------------------------------------------------------------
// One-click import support
//------------------------------------------------------------------------

// Set theme specific importer options
if ( ! function_exists( 'themerex_importer_set_options' ) ) {
	add_filter( 'trx_addons_filter_importer_options', 'themerex_importer_set_options', 9 );
	function themerex_importer_set_options( $options=array() ) {
		if ( is_array( $options ) ) {
			// Save or not installer's messages to the log-file
			$options['debug'] = false;
			// Prepare demo data
			if ( is_dir( THEMEREX_THEME_PATH . 'demo/' ) ) {
				$options['demo_url'] = THEMEREX_THEME_PATH . 'demo/';
			} else {
				$options['demo_url'] = esc_url( themerex_get_protocol().'://demofiles.themerex.net/alliance/' ); // Demo-site domain
			}

			// Required plugins
			$options['required_plugins'] =  array(
				'booked',
				'bbpress',
				'content_timeline',
				'js_composer',
				'essential-grid'
			);

			$options['theme_slug'] = 'alliance';

			// Set number of thumbnails to regenerate when its imported (if demo data was zipped without cropped images)
			// Set 0 to prevent regenerate thumbnails (if demo data archive is already contain cropped images)
			$options['regenerate_thumbnails'] = 3;
			// Default demo
			$options['files']['default']['title'] = esc_html__( 'Alliance Demo', 'alliance' );
			$options['files']['default']['domain_dev'] = esc_url(themerex_get_protocol().'://alliance.upd.themerex.net'); // Developers domain
			$options['files']['default']['domain_demo']= esc_url(themerex_get_protocol().'://alliance.themerex.net'); // Demo-site domain

		}
		return $options;
	}
}


/* Include framework core files
------------------------------------------------------------------- */

require_once( get_template_directory().'/fw/loader.php' );
?>